/*
 * logan_gpio.c
 *
 *  Created on: Aug 24, 2024
 *      Author: longan
 */

#include "ch32v20x.h"
#include "logan_gpio.h"

/*********************************************************************
 * @fn      logan_gpio_init
 *
 * @brief   GPIO IN/OUT MODE init
 *
 * @param   PORTX - where x can be (A..G) to select the GPIO peripheral.
 *          GPIO_Pin - specifies the port bit to read.
 *             This parameter can be GPIO_Pin_x where x can be (0..15).
 *          MODE -      GPIO_Mode_IN_FLOATING = 0x04,
                        GPIO_Mode_IPD = 0x28,
                        GPIO_Mode_IPU = 0x48,
                        GPIO_Mode_Out_OD = 0x14,
                        GPIO_Mode_Out_PP = 0x10,
 *
 * @return  none
 */
void logan_gpio_init(u16 PORTX,u16 GPIO_Pin,u8 MODE)
{
    GPIO_InitTypeDef GPIO_InitStructure = {0};

    //Enable GPIO CLK
    switch (PORTX) {
        case PORTA:
            RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);
            break;
        case PORTB:
            RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);
            break;
        case PORTC:
            RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE);
            break;
        case PORTD:
            RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOD, ENABLE);
            break;
        case PORTE:
            RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOE, ENABLE);
            break;
    }

    //init GPIO_InitStructure
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin;
    GPIO_InitStructure.GPIO_Mode = MODE;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(((GPIO_TypeDef *)(APB2PERIPH_BASE + PORTX)), &GPIO_InitStructure);
}

/*********************************************************************
 * @fn      logan_get_pin
 *
 * @brief   get gpio input
 *
 * @param   PORTX - where x can be (A..G) to select the GPIO peripheral.
 *          GPIO_Pin - specifies the port bit to read.
 *             This parameter can be GPIO_Pin_x where x can be (0..15).
 *
 * @return  none
 */
u8 logan_get_pin(u16 PORTX,u16 GPIO_Pin)
{
    return GPIO_ReadInputDataBit(((GPIO_TypeDef *)(APB2PERIPH_BASE + PORTX)), GPIO_Pin);
}

void logan_set_pin(u16 PORTX,u16 GPIO_Pin,u8 value)
{
    GPIO_WriteBit(((GPIO_TypeDef *)(APB2PERIPH_BASE + PORTX)), GPIO_Pin, value);
}

