/*
 * logan_i2c2.c
 *
 *  Created on: Aug 24, 2024
 *      Author: longan
 */
#include "logan_i2c2.h"
#include "ch32v20x.h"
#include "debug.h"

void logan_IIC2_Init(u32 bound, u16 address)
{
    GPIO_InitTypeDef GPIO_InitStructure={0};
    I2C_InitTypeDef I2C_InitTSturcture={0};
    //I2C2 PB10 PB11
    RCC_APB2PeriphClockCmd( RCC_APB2Periph_GPIOB, ENABLE );
    RCC_APB1PeriphClockCmd( RCC_APB1Periph_I2C2, ENABLE );

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_OD;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init( GPIOB, &GPIO_InitStructure );

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_OD;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init( GPIOB, &GPIO_InitStructure );
    //I2C_ClockSpeed 100K 400K
    I2C_InitTSturcture.I2C_ClockSpeed = bound;
    I2C_InitTSturcture.I2C_Mode = I2C_Mode_I2C;
    //I2C_DutyCycleģʽʱռձ
    I2C_InitTSturcture.I2C_DutyCycle = I2C_DutyCycle_16_9;
    I2C_InitTSturcture.I2C_OwnAddress1 = address;
    I2C_InitTSturcture.I2C_Ack = I2C_Ack_Enable;
    I2C_InitTSturcture.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;
    I2C_Init( I2C2, &I2C_InitTSturcture );
    I2C_DMACmd( I2C2, ENABLE );
    I2C_Cmd( I2C2, ENABLE );
}

void logan_i2c2_tx(u8 addr)
{
    while( I2C_GetFlagStatus( I2C2, I2C_FLAG_BUSY ) != RESET );
    I2C_GenerateSTART( I2C2, ENABLE );
    while( !I2C_CheckEvent( I2C2, I2C_EVENT_MASTER_MODE_SELECT ) );
    I2C_Send7bitAddress( I2C2, 0xA0, I2C_Direction_Transmitter );
    while( !I2C_CheckEvent( I2C2, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED ) );
    DMA_Cmd( DMA1_Channel4, ENABLE );
    while( !I2C_CheckEvent( I2C2, I2C_EVENT_MASTER_BYTE_TRANSMITTED ) );

    I2C_GenerateSTOP( I2C2, ENABLE );
}

void logan_send_addr(I2C_TypeDef *I2Cx, u8 slave_add, u8 I2C_Direction) {
    if (I2C_Direction != I2C_Direction_Transmitter) {
        slave_add = (slave_add << 1) | 1;
    } else {
        slave_add = (slave_add << 1) & 0xFFFE;
    }

    I2Cx->DATAR = slave_add;
}
void logan_i2c2_rx(u16 slave_add, u16 reg)
{


    while( I2C_GetFlagStatus( I2C2, I2C_FLAG_BUSY ) != RESET );
    I2C_GenerateSTART( I2C2, ENABLE);

    while( !I2C_CheckEvent( I2C2, I2C_EVENT_MASTER_MODE_SELECT ) );
    I2C_Send7bitAddress( I2C2, slave_add, I2C_Direction_Transmitter);

    while( !I2C_CheckEvent( I2C2, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED ) );
#if (Address_Lenth  == Address_8bit)
    I2C_SendData( I2C2, (u8) (reg & 0x00FF));
    while( !I2C_CheckEvent( I2C2, I2C_EVENT_MASTER_BYTE_TRANSMITTED ) );


#elif (Address_Lenth  == Address_16bit)
    I2C_SendData( I2C1, (u8)(reg>>8) );
    while( !I2C_CheckEvent( I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED ) );

    I2C_SendData( I2C1, (u8)(reg&0x00FF) );
    while( !I2C_CheckEvent( I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED ) );

#endif

    I2C_GenerateSTART( I2C2, ENABLE);

    while( !I2C_CheckEvent( I2C2, I2C_EVENT_MASTER_MODE_SELECT ) );
    I2C_Send7bitAddress( I2C2, slave_add, I2C_Direction_Receiver);

    while( !I2C_CheckEvent( I2C2, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED ) );
    while( I2C_GetFlagStatus( I2C2, I2C_FLAG_RXNE ) ==  RESET )
//    I2C_AcknowledgeConfig( I2C2, DISABLE );
//    if (bufsize==1) {
//        I2C_AcknowledgeConfig( I2C2, DISABLE ); // 룬ǰرԶӦACK
//    }
    DMA_Cmd( DMA1_Channel5, ENABLE );

}

u8 poll_i2c_devices(u8* device_list)
{
    u8 i=1;
    u8 j=0;

    for(i=1;i<127;i++)
    {
        while( I2C_GetFlagStatus( I2C2, I2C_FLAG_BUSY ) != RESET );
        I2C_GenerateSTART( I2C2, ENABLE);

        while( !I2C_CheckEvent( I2C2, I2C_EVENT_MASTER_MODE_SELECT ) );
        logan_send_addr( I2C2, i, I2C_Direction_Transmitter);
        Delay_Us(100);

        I2C_GenerateSTOP( I2C2, ENABLE);

        if (I2C_GetFlagStatus(I2C2,I2C_FLAG_TXE)) {
            device_list[j++]=i;
        }

    }
    I2C_ITConfig( I2C2, I2C_IT_EVT, DISABLE );
    return j;
}

